/**
http://homes.cs.washington.edu/~djg/teachingMaterials/spac/sophomoricParallelismAndConcurrency.pdf

*/
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveAction;
import java.util.Random;

class SumArray extends RecursiveAction
{
   static int SEQUENTIAL_CUTOFF = 1_000;

   int lo;  // fields for communicating inputs
   int hi;
   int[] arr;
   int ans = 0; // for communicating result

   SumArray(int[] a, int l, int h)
   {
      arr=a; lo=l; hi=h;
   }

   protected void compute()
   {
      if (hi - lo <= SEQUENTIAL_CUTOFF)
      {
         for (int i = lo; i < hi; ++i)
            ans += arr[i];
      }
      else
      {
         SumArray left  = new SumArray(arr,lo,(hi+lo)/2);
         SumArray right = new SumArray(arr,(hi+lo)/2,hi);

         left.fork();     // asynchronous, put left item in the work queue
         right.compute(); // synchronous, continue working into the right item
         left.join();     // whenever some worker thread completes left item
         ans = left.ans + right.ans;
      }
   }
}


class C
{
 //static final ForkJoinPool fjPool = new ForkJoinPool();  // needed by Java 7

   static int sum(int[] array)
   {
      SumArray t = new SumArray(array, 0, array.length);

      // Put item t into the work queue.
      ForkJoinPool.commonPool().invoke(t); // Java 8
    //fjPool.invoke(t);                    // Java 7
      return t.ans;
   }
}


public class Section_3_4a
{
   public static final int PROBLEM_SIZE = 100_000_000;
   public static int numbers[] = new int[PROBLEM_SIZE];

   public static void main(String[] args)
   {
      Random generator = new Random();
      for (int i = 0; i < numbers.length; i++)
         numbers[i] = generator.nextInt();

      long startTime;
      long stopTime;

      long result = 0;

      SumArray.SEQUENTIAL_CUTOFF = 5000;  // BE SURE TO EXPERIMENT WITH THIS!
      System.out.println("Starting Fork/Join Pool...");
      startTime = System.currentTimeMillis();
         result = C.sum(numbers);
      stopTime = System.currentTimeMillis();
      System.out.println(" Fork-join sum: " + result);
      System.out.println(" Fork-join solution time: " + (stopTime - startTime));
      System.out.println("ForkJoinPool: " + ForkJoinPool.commonPool().toString());
    //System.out.println("ForkJoinPool: " + Main.fjPool.toString());
   }
}
